/* Network Trackerd
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@ab.nom.tr>
http://people.sabanciuniv.edu/alib
http://www.ab.nom.tr
2004.03.22
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/
#include "tracker.h"

#ifdef HAVE_CONFIG_H
    #include <config.h>
#endif

int ldapsearch_main(char *filter, char *sonuc, struct ayr *ayarlar);


#ifdef HAVE_LDAP
#include "portable.h"
#include <stdio.h>
#include <ac/stdlib.h>
#include <ac/ctype.h>
#include <ac/string.h>
//#include <ac/unistd.h>
#include <ac/errno.h>
#include <sys/stat.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_IO_H
#include <io.h>
#endif
#include <ldap.h>
#include "ldif.h"
#include "lutil.h"
#include "lutil_ldap.h"
#include "ldap_defaults.h"
#include "ldap_log.h"
#include "common.h"
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// LDAP attribute'leri
#define MAC "macaddress"
#define LDAPPORT 0
#define SORTATTR "uid"
#define VERBOSE 0

static void print_entry LDAP_P((LDAP *ld, LDAPMessage *entry));
static int write_ldif(int type, char *name, char* value, ber_len_t vallen);
int dosearch LDAP_P((LDAP *ld, int scope, char *filtpatt, char *value,
	char **attrs, int attrsonly, LDAPControl **sctrls, LDAPControl **cctrls,
	struct timeval *timelimit, int	sizelimit));
int ldapsearch_main(char *filter, char *sonuc, struct ayr *ayarlar);

char BASE[255];
char deger[100];

int ldap_uid_ekle(struct ayr *ayarlar)
{
    MYSQL mysql, mysql_update;
    MYSQL_ROW mysqlrow;
    MYSQL_RES *mysqlresult;
    int num_macs=0, uid_number=0, i, ls;
    char query[500], uid[100], filter[100];

    mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_INFO, "databaze_yaz: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        return 0;
    }

    mysql_init(&mysql_update);
    if(!mysql_real_connect(&mysql_update, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_INFO, "databaze_yaz: Failed to connect to database: Error: %s\n", mysql_error(&mysql_update));
        return 0;
    }

    sprintf(query, "select mac from macs where time='%s'", ayarlar->LastUpdateTime);
    #ifdef debug
        syslog(LOG_INFO, query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "MySQL Error:%s. MAC update query calisamadi ! Query=%s", mysql_error(&mysql), query);
    }
    else { //query applied
        mysqlresult = mysql_store_result(&mysql);
        num_macs = mysql_num_rows(mysqlresult);
        syslog(LOG_INFO, "Database'deki MAC sayisi:%d", num_macs);
        for(i=0; i<num_macs; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
            strcpy(uid, "uid"); sprintf(filter, "macaddress=1,6,%s", mysqlrow[0]);
            //syslog(LOG_INFO, "ldap'ta aranacak mac:%s-", filter);
            ls=ldapsearch_main(filter, uid, ayarlar);
            //printf("ldapsearch result:%d\n", ls);
            if( ls==-1 || !ls) {
                syslog(LOG_INFO, "ldap'ta mac:%s bulunamadi", mysqlrow[0]);
                strcpy(uid, "notfound");
             }
             //syslog(LOG_INFO, "ldapta bulunan uid:%s, mac:%s", uid, mysqlrow[0]);
             sprintf(query, "update macs set uid='%s' where mac='%s'", uid, mysqlrow[0]);
             #ifdef debug
                syslog(LOG_INFO, query);
             #endif
             if( mysql_real_query(&mysql_update, query, strlen(query)) ) {
                syslog(LOG_INFO, "MySQL Error:%s. uid update query calisamadi ! Query=%s", mysql_error(&mysql_update), query);
             }
             else { //query applied
                uid_number++;
                syslog(LOG_INFO, "uid update edildi. Query=%s", query);
             }
        }
        if( mysqlresult ) mysql_free_result(mysqlresult);
    }
    mysql_close(&mysql); mysql_close(&mysql_update);
    syslog(LOG_INFO, "ldap'tan %d uid update edildi.", uid_number);
    return 1;
}

int ldapsearch_main(char *filter, char *sonuc, struct ayr *ayarlar)
{
    char *attrs[18];
    int	rc, i, scope, deref, attrsonly; //first
    int	authmethod, version, want_bindpw;
    LDAP *ld=NULL;

    attrsonly = want_bindpw = 0;
    deref = version = -1;
    scope = LDAP_SCOPE_SUBTREE;
    authmethod = -1;

    attrs[0]="uid";
    //attrs[0]=MAC;
    attrs[1]=NULL;

    version=LDAP_VERSION3;
    authmethod=LDAP_AUTH_SIMPLE;

    #ifdef SIGPIPE	
        (void) SIGNAL(SIGPIPE, SIG_IGN);
    #endif
    
    strcpy(BASE, ayarlar->ldapbase);

    ld = ldap_init(ayarlar->ldaphost, LDAPPORT);
    if(!ld) {
	    perror("ldapsearch: ldap_init");
	    return EXIT_FAILURE;
    }

    /* referrals */
    if (ldap_set_option(ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF ) != LDAP_OPT_SUCCESS ) {
	    printf("Could not set LDAP_OPT_REFERRALS %s\n", "off" );
	    return EXIT_FAILURE;
    }

    if( ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) != LDAP_OPT_SUCCESS ) {
	    printf("Could not set LDAP_OPT_PROTOCOL_VERSION %d\n", version);
	    return EXIT_FAILURE;
    }

    if(ldap_bind_s(ld, ayarlar->ldapmanager, ayarlar->ldapmanagerpass, authmethod) != LDAP_SUCCESS ) {
	    ldap_perror(ld, "ldap_bind");
	    return(EXIT_FAILURE);
    }
	
    if ( VERBOSE) {
	    printf("Filter: %s\nrequesting: ", filter);
        if(!attrs) printf("ALL");
	    else for(i=0; attrs[i]; ++i) printf("%s ", attrs[i]);
        printf("\n" );
    }

    rc = dosearch(ld, scope, filter, NULL, attrs, attrsonly, NULL, NULL, NULL, -1);
    ldap_unbind(ld);
    strcpy(sonuc, deger);
    return(rc);
}


int dosearch(LDAP *ld, int scope, char *filtpatt, char *value,
	char **attrs, int attrsonly, LDAPControl **sctrls, LDAPControl **cctrls,
	struct timeval *timelimit, int sizelimit )
{
    int	rc, nentries, nreferences, nextended, npartial, mestip;
    LDAPMessage	*res, *msg;
    ber_int_t msgid;

    rc = ldap_search_ext(ld, BASE, scope, filtpatt, attrs, attrsonly,
	sctrls, cctrls, timelimit, sizelimit, &msgid );

    if(rc != LDAP_SUCCESS) {
        printf("ldap_search_ext: %s (%d)\n",
	    ldap_err2string(rc), rc);
        return(rc);
    }

    nentries = nreferences = nextended = npartial = 0;
    res = NULL;

    while ((rc = ldap_result( ld, LDAP_RES_ANY, SORTATTR ? LDAP_MSG_ALL : LDAP_MSG_ONE, NULL, &res )) > 0 ) {
	    if( SORTATTR )
    	    (void)ldap_sort_entries(ld, &res, SORTATTR, strcasecmp);

	    for(msg = ldap_first_message(ld, res); msg != NULL; msg = ldap_next_message(ld, msg) ) {
	        mestip=ldap_msgtype(msg);
	        switch( ldap_msgtype(msg) ) {
		        case LDAP_RES_SEARCH_ENTRY:
		            nentries++;
		            print_entry(ld, msg);
		            break;
		        case LDAP_RES_SEARCH_RESULT:
		            //printf("Bulunan Kayit Sayisi:%d\n", nentries);
                    return nentries;
	        }
	    }
	    ldap_msgfree(res);
    }

    if (rc==-1) {
	    ldap_perror(ld, "ldap_result");
	    return(rc);
    }
    return nentries;
}


static void print_entry(LDAP *ld, LDAPMessage *entry)
{
    char *a;
    int i;
    BerElement *ber = NULL;
    struct berval **bvals;
        
    for(a=ldap_first_attribute( ld, entry, &ber ); a != NULL; a=ldap_next_attribute( ld, entry, ber ) ) {
	    if( (bvals=ldap_get_values_len(ld, entry, a)) != NULL ) {
	    for (i=0; bvals[i]; i++) 
		    write_ldif(LDIF_PUT_VALUE, a, bvals[i]->bv_val, bvals[i]->bv_len);		
	    ber_bvecfree(bvals);
	    }
    }
    if(ber) ber_free(ber, 0);
}

static int write_ldif(int type, char *name, char* value, ber_len_t vallen)
{
    char *ldif, *buf;
    
    if( (ldif=ldif_put(type, name, value, vallen)) == NULL) return(-1);
    ldif[strlen(ldif)-1]=(char)NULL;
    buf=strstr(ldif, ": ");
    //printf("ldif:%s,uz=%d\n", &buf[2], strlen(&buf[2]));
    strcpy(deger, &buf[2]);
    ber_memfree(ldif);
    return 0;
}	
#endif    
