#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <syslog.h>

#include "trackerdbgui.h"
#include "interface.h"
#include "support.h"

/* Menus in main window : */
void on_file_exit(GtkMenuItem *menuitem, gpointer user_data)
{
   syslog(LOG_INFO,"on_exit_activate called\n");
   gtk_main_quit();
}

void on_help_about(GtkMenuItem *menuitem, gpointer user_data)
{
	ShowMessageBox(	" trackerdbgui Version: "VERSION"\n"\
					"\n Author: Ali BASEL <alib@sabanciuniv.edu>\n"\
					" http://www.basel.name.tr\n\n"\
					"This pragram is a total modification of the GPL licensed very nice gequel program.\n"\
					"trackerdbgui uses GTK2 and gtktreeview instead of deprecated clist.\n"\
					"Modified to edit only trackerdb database.\n"\
					"Original Author: Kees Lemmens <lemmens at users.sourceforge.net>\n"\
					"http://sourceforge.net/projects/gequel", (Params *)user_data);
}

void on_help_usage(GtkMenuItem *menuitem, gpointer user_data)
{
   ShowMessageBox(	"\n trackerdbgui helps adding/editing/modifying records for the trackerdb tables on the MySQL database.\n"\
					" Master tables or single tables are located above and detail tables are located below.\n\n"\
					" When a record of a master table is choosen, all of its detail records are shown below.\n"\
					" - Insert function creates a new record for the currently selected table.\n"\
	       			" - Delete function deletes the selected record in the table; if the table has a master-detail relation, all of the detail records are also deleted. The record numbers deleted from the tables are shown in the status bar.\n"\
					" - Show Records function retrieves all the records of the currently selected table.\n"
					" If the selected table is a detail table then all of its records also are shown.\n"
					" - Pressing on the 'Insert' key on the keyboard, adds a detail record for the selected record on a master table.\n"\
					" - Clicking on a column, shows the table ordered by that column. The sort order can be changed by clicking on the same column again.\n"\
					" - The records can be edited by selecting a field and pressing the 'enter' or 'space' key.\n"\
					" - Detail records on the bottom can be hidden by pressing the 'ctrl' key twice, and can be shown the same way.\n"\
					" - Search function isn't functioning.\n",(Params *)user_data);
}

/* Tab'a gecildiginde o tab'a ait xx_button_clicked functions button'lara baglanir */
void on_switch_page(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer user_data)
{
    Params *par=(Params *)user_data;
    TabInfo *ti=par->TabInfoStructs[page_num];
	
	syslog(LOG_INFO, "notebook page has been switched to page#%d\n", page_num);
	update_statusbar(par, "Table: %s", ti->table);
	par->last_selected_gtktreeview=ti;
	if( ti->show_details ) {
		gtk_toggle_button_set_active( (GtkToggleButton*)GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")), TRUE );
		gtk_widget_show( par->detailframe );
	}
	else {
		gtk_toggle_button_set_active( (GtkToggleButton*)GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")), FALSE );
		gtk_widget_hide( par->detailframe );	
	}
	if( ti->DetailTableNo==-1 ) gtk_widget_hide( GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")) );
	else gtk_widget_show( GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")) );
}

/* Attach the widgets we need in the code to the Params structure */
void storeWidgets(GtkWindow *main, Params *p)
{
   p->notebook = GTK_WIDGET(lookup_widget((GtkWidget *)main, "Main_Notebook"));
   p->detailsnotebook = GTK_WIDGET(lookup_widget((GtkWidget *)main, "Details_Notebook"));
   p->detailframe=GTK_WIDGET(lookup_widget((GtkWidget *)main, "ModulesGtkFrame"));
}

void connectMainWindowSignals(GtkWindow *main, Params *par)
{
	syslog(LOG_INFO,"connectMainWindowSignals entered\n");
   /* Menus : */
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"file_exit")), "activate", G_CALLBACK(on_file_exit), par);
   	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"edit_ShowRecords")), "activate", G_CALLBACK(on_edit_show), par);  
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"edit_DeleteRecord")), "activate", G_CALLBACK(on_edit_delete), par);  
   	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"edit_InsertRecord")), "activate", G_CALLBACK(on_edit_insert), par);
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"edit_SearchRecords")), "activate", G_CALLBACK(on_edit_search), par);
   	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"help_about")), "activate", G_CALLBACK(on_help_about), par);
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"help_usage")), "activate", G_CALLBACK(on_help_usage), par);

	   
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"ShowRecordsGtkButton")), "clicked", G_CALLBACK(on_edit_show), (gpointer)par);
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"InsertRecordGtkButton")), "clicked", G_CALLBACK(on_edit_insert), (gpointer)par);
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"DeleteRecordGtkButton")), "clicked", G_CALLBACK(on_edit_delete), (gpointer)par);
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)main,"SearchRecordsGtkButton")), "clicked", G_CALLBACK(on_edit_search), (gpointer)par);

	g_signal_connect(GTK_OBJECT(lookup_widget((GtkWidget *)main,"Main_Notebook")), "switch-page", G_CALLBACK(on_switch_page), (gpointer)par);

	g_signal_connect(GTK_OBJECT(lookup_widget((GtkWidget *)main,"DetailsGtkCheckButton")), "toggled", G_CALLBACK(on_DetailsGtkCheckButton_toggled), (gpointer)par);
	
}


void trackerdbguiInit(Params *p, int argc, char *argv[])
{	
	enum tablolar { MAIN, AYARLAR, STATS, MACS, STOCKMAIN, MODULES, STOCKMODULES };
	TabInfo *ti;
	struct ayr ayarlar;
	char *pass=NULL;
	
        openlog("trackerdbgui", LOG_NOWAIT,  LOG_LOCAL4);

	syslog(LOG_INFO,"trackerdbgui started");
	
	if( !config_oku(&ayarlar) ) {
		puts("Look at syslog message file for the error !\n");
		exit(1);
	}
	
	
	strncpy(p->server, ayarlar.sqlhost, sizeof(p->server)-1);
	strncpy(p->database, ayarlar.sqldb, sizeof(p->database)-1);
	
	/* If user has supplied a username on the startup as an argument
	   then get user's password and use them when connecting to mysql server 
	   If not, use the default username/password defined in the config file.
	   The latter, should have only read access on the database.
	*/
	if( argc==2 ) {
		pass=getpass("Password: ");
		#ifdef DEBUG
		syslog(LOG_INFO, "argc:%d, argv[1]:%s, pass=%s\n", argc, argv[1], pass);
		#endif
		strncpy(p->user, argv[1], sizeof(p->user)-1);
	        strncpy(p->pwd, pass, sizeof(p->pwd)-1);
		if(pass)free(pass);
		syslog(LOG_INFO, "USER: %s STARTED TO USE TRACKERDBGUI", p->user);
	}
	else {
		strncpy(p->user, ayarlar.sqluser, sizeof(p->user)-1);
		strncpy(p->pwd, ayarlar.sqlpass, sizeof(p->pwd)-1);
		syslog(LOG_INFO, "DEFAULT USER: %s STARTED TO USE TRACKERDBGUI", p->user);
	}


	p->dbsock=mysql_init(NULL);
	if( !p->dbsock || !mysql_real_connect(p->dbsock, p->server, p->user, p->pwd, p->database, 0, NULL, 0) ) {
		showSqlError(p, "Error connecting to server");
		exit(1);
	}

		
    /***************************************************/
	/* table name: "Main" ve GtkTreeView object ismi: MainGtkTreeView */
	ti=p->TabInfoStructs[MAIN]=(TabInfo *)newTabInfostruct(p, "Main", "MainGtkTreeView");
    if( !p->TabInfoStructs[MAIN] ) {
        syslog(LOG_INFO, "appInit: Couldn't create MainTableTabInfoStruct\n");
        exit(0);
    }	
	ti->notebook_page_num=0;
	/* Call record_selected_showdetail function when a row selected for that table */
	ti->gtktreeview_line_selected_function=record_selected;
	ti->show_details=1; /* show details by default */
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
    /* "key-press-event" signal'i baglaniyor*/
	g_signal_connect(ti->gtktreeview, "key-press-event", (GCallback) gtktreeview_onButtonPressed, ti);
	p->last_selected_gtktreeview=ti; /* This gtktreeview will be seen on screen initially, so set this */
	/***************************************************/
	
	
	/***************************************************/
    /* table name: "Ayarlar", AyarlarGtkScrolledWindow */
	ti=p->TabInfoStructs[AYARLAR]=(TabInfo *)newTabInfostruct(p, "Ayarlar", "AyarlarGtkTreeView");
    if( !p->TabInfoStructs[AYARLAR] ) {
        syslog(LOG_INFO, "appInit: Couldn't create AyarlarTableTabInfoStruct\n");
        exit(0);
    }
	ti->notebook_page_num=1;
	ti->DetailTableNo=-1; /* no master-detail relation */
	ti->gtktreeview_line_selected_function=record_selected;
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
	/* "key-press-event" signal'i baglaniyor*/
	g_signal_connect(ti->gtktreeview, "key-press-event", (GCallback) gtktreeview_onButtonPressed, ti);
	/***************************************************/

	
	/***************************************************/
    /* table name: "Stats", StatsGtkScrolledWindow */
	ti=p->TabInfoStructs[STATS]=(TabInfo *)newTabInfostruct(p, "Stats", "StatsGtkTreeView");
    if( !p->TabInfoStructs[STATS] ) {
        syslog(LOG_INFO, "appInit: Couldn't create StatsTableTabInfoStruct\n");
        exit(0);
    }
	ti->notebook_page_num=1;
	ti->DetailTableNo=-1; /* no master-detail relation */
	ti->gtktreeview_line_selected_function=record_selected;
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
	/* "key-press-event" signal'i baglaniyor*/
	g_signal_connect(ti->gtktreeview, "key-press-event", (GCallback) gtktreeview_onButtonPressed, ti);
	/***************************************************/

	    
	
	/***************************************************/
    /* table name: "macs", MacsGtkScrolledWindow */
	ti=p->TabInfoStructs[MACS]=(TabInfo *)newTabInfostruct(p, "macs", "MacsGtkTreeView");
    if( !p->TabInfoStructs[MACS] ) {
        syslog(LOG_INFO, "appInit: Couldn't create MacsTableTabInfoStruct\n");
        exit(0);
    }
	ti->notebook_page_num=2; /* "MacsGtkTreeView" is the 3. page in the main notebook */
	ti->DetailTableNo=-1; /* no master-detail relation */
	ti->gtktreeview_line_selected_function=record_selected;
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
	/* "key-press-event" signal'i baglaniyor*/
	g_signal_connect(ti->gtktreeview, "key-press-event", (GCallback) gtktreeview_onButtonPressed, ti);
	/***************************************************/
    
	/***************************************************/
	/* table name: "StockMain" ve GtkTreeView object ismi: StockMainGtkTreeView */
	ti=p->TabInfoStructs[STOCKMAIN]=(TabInfo *)newTabInfostruct(p, "StockMain", "StockMainGtkTreeView");
    if( !p->TabInfoStructs[STOCKMAIN] ) {
        syslog(LOG_INFO, "appInit: Couldn't create StockMainTableTabInfoStruct\n");
        exit(0);
    }	
	ti->notebook_page_num=3; /* "StockMainGtkTreeView" is the 3. page in the main notebook */
	/* Call record_selected_showdetail function when a row selected for that table */
	ti->gtktreeview_line_selected_function=record_selected;
	ti->show_details=1; /* show details by default */
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
    /* "key-press-event" signal'i baglaniyor*/
	g_signal_connect(ti->gtktreeview, "key-press-event", (GCallback) gtktreeview_onButtonPressed, ti);
	p->last_selected_gtktreeview=ti; /* This gtktreeview will be seen on screen initially, so set this */
	/***************************************************/
	
	
	
	/***************************************************/                    
    /* table name: "Modules", ModulesGtkSTreeViewName */
    ti=p->TabInfoStructs[MODULES]=(TabInfo *)newTabInfostruct(p, "Modules", "ModulesGtkTreeView");
    if( !p->TabInfoStructs[MODULES] ) {
        syslog(LOG_INFO, "appInit: Couldn't create TableTabInfoStruct[MODULES]\n");
        exit(0);
    }
	ti->notebook_page_num=0; /* "ModulesGtkTreeView" is the 0. page in the details notebook */
	ti->gtktreeview_line_selected_function=record_selected;
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
	/***************************************************/
	
	/***************************************************/                    
    /* table name: "StockModules", StockModulesGtkSTreeViewName */
    ti=p->TabInfoStructs[STOCKMODULES]=(TabInfo *)newTabInfostruct(p, "StockModules", "StockModulesGtkTreeView");
    if( !p->TabInfoStructs[STOCKMODULES] ) {
        syslog(LOG_INFO, "appInit: Couldn't create TableTabInfoStruct[STOCKMODULES]\n");
        exit(0);
    }
	ti->notebook_page_num=1; /* "StockModulesGtkTreeView" is the 1. page in the details notebook */
	ti->gtktreeview_line_selected_function=record_selected;
	g_signal_connect( ti->gtktreeview, "cursor-changed", G_CALLBACK(ti->gtktreeview_line_selected_function), (gpointer)ti);
	/***************************************************/
	
	set_master_detail_relation(p, "Main", "Modules", "MainSerialNo");
	set_master_detail_relation(p, "StockMain", "StockModules", "MainSerialNo");
}
